//---------------------------------------------------------------------------

#include <vcl.h>
#include <math.h>

#pragma hdrstop

#include "Unit1.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TForm1 *Form1;
//---------------------------------------------------------------------------
__fastcall TForm1::TForm1(TComponent* Owner)
        : TForm(Owner)
{
}
//---------------------------------------------------------------------------



const
  N = 100; // total number of atoms
const cellsize_x = 2;
  const cellsize_y = 2;
  const float potential_d = 0.2;
  const float potential_e = 100;
  const float potential_a = potential_e*potential_d*potential_d*potential_d*potential_d*potential_d*potential_d*potential_d*potential_d*potential_d*potential_d*potential_d*potential_d;
  const float potential_b = 2*potential_e*potential_d*potential_d*potential_d*potential_d*potential_d*potential_d;
  const border_k = 2*500*potential_e;

  const float initial_x1 = 0.0; const float initial_x2 = 2.0;
  const float initial_y1 = 0.0; const float initial_y2 = 2.0;
  const float initial_dt = 0.000005;
  const float initial_kinetic = 80;

  const float mAr=6.69e-26; const float v_unit=14.362;  //sqrt(1.38e-23/mAr);
  const float L_unit=3.405e-10/potential_d;
  const float t_unit=L_unit/v_unit;

  const Graph_N = 300;
  // for g(r) calculations
  //const float GR_max_R = potential_d * 8;
  const GR_N = Graph_N;
  //const float GR_step = 2/(GR_N-1);
  //const GR_shift = 0;
 // const float GR_strip_width = potential_d/4.0;
  // for n(e) calculations
  const E_points = Graph_N;
  const E_Slices = 10;

  // potential modes
const  Mode_Fixed = 1;
const  Mode_Periodic = 2;

struct Atom{
  double x,y;
  double vx,vy;
};

struct Energy{
  double u, k;
};

Atom initial_atoms[N+1], atoms[N+1];
double initial_energy, impulse_x, impulse_y, total_time;
double dddx,dddy;
Atom acel_1[N+1],acel_2[N+1],acel_3[N+1], temp_atoms[N+1];
Energy  EA[N+1];

int mode;  int tim;  float c[100]; double gr[100]; double GR[100];

double rand_double(double min, double max)
  { const granularity = 30000;
   double zypsyksy = min+((max-min)*random(granularity))/(granularity-1);
    return zypsyksy;
  }

// ----------------------- Basic Force & Potential ----------------------- //

double potential_direct(double dx,double dy)
{
double d2,d6;
d2 = double(1.0/(dx*dx+dy*dy));
d6 = double(d2*d2*d2);
double zypsyksy = (potential_a*d6 - potential_b)*d6;
return zypsyksy;
}

double border_potential(double x, double y)
{
double p;
p = 0;

if (x > cellsize_x) p = p + border_k*(x-cellsize_x)*(x-cellsize_x)/2.0;
  else if (x < 0) p = p + border_k*x*x/2.0;
if (y > cellsize_y) p = p + border_k*(y-cellsize_y)*(y-cellsize_y)/2.0;
  else if (y < 0)  p = p + border_k*y*y/2.0;
return p;
}

void force_direct(double dx, double dy,double &fx,double &fy)
{
 double d2,d6,f;
 dddx=dx; dddy=dy;
 d2 = 1.0/(dddx*dddx+dddy*dddy);
 d6 = d2*d2*d2;
 f = (12*potential_a*d6 - 6*potential_b)*d6*d2;
 fx = f*dx;
 fy = f*dy;
}

void border_force(double x, double y,double  &fx,double &fy)
{
  if (x > cellsize_x) fx = -border_k*(x-cellsize_x);
    else if (x < 0) fx = -border_k*x;
      else fx = 0;
  if (y > cellsize_y) fy = -border_k*(y-cellsize_y);
    else if (y < 0) fy = -border_k*y;
      else fy = 0;
}

double potential_periodic(double dx,double dy)
{
   if (dx<0) dx=-dx;
   if (dy<0) dy=-dy;
   if (dx > cellsize_x*0.5) dx = dx - cellsize_x;
   if (dy > cellsize_y*0.5) dy = dy - cellsize_y;
   double zypsyksy= potential_direct(dx, dy);
   return zypsyksy;
}

void force_periodic(double dx,double dy,double &fx,double &fy)
{
 double ddx;
 if (dx<=0) ddx=-dx; else ddx=dx;
 double ddy;
 if (dy<=0) ddy=-dy; else ddy=dy;
 if (ddx > cellsize_x*0.5) ddx = ddx - cellsize_x;
 if (ddy > cellsize_y*0.5) ddy = ddy - cellsize_y;
 force_direct(ddx, ddy, fx, fy);
 if (dx < 0) fx = -fx;
 if (dy < 0) fy = -fy;
}

// ----------------------- Forces & Potentials ----------------------- //

void calc_derivatives(Atom a[], Atom f[])
{
  int i,j;
  double  fx, fy;
  for (i=1;i<=N;i++)
    {
    f[i].vx = 0;
    f[i].vy = 0;
    }
  if (mode == Mode_Fixed)     // fixed border
    {
    for (i=1;i<=N-1;i++)
      for (j=i+1;j<=N;j++)
        {
        force_direct(a[i].x-a[j].x, a[i].y-a[j].y, fx, fy);
        f[i].vx = f[i].vx + fx;
        f[i].vy = f[i].vy + fy;
        f[j].vx = f[j].vx - fx;
        f[j].vy = f[j].vy - fy;
        }
    for (i=1;i<=N;i++)
      {
      border_force(a[i].x, a[i].y, fx, fy);
      f[i].vx = f[i].vx + fx;
      f[i].vy = f[i].vy + fy;
      }
    }
    else   // periodic border
    {
    for (i=1;i<=N-1;i++)
      for (j=i+1;j<=N;j++)
      {
      force_periodic(a[i].x-a[j].x, a[i].y-a[j].y, fx, fy);
      f[i].vx = f[i].vx + fx;
      f[i].vy = f[i].vy + fy;
      f[j].vx = f[j].vx - fx;
      f[j].vy = f[j].vy - fy;
      }
  }
}

// -------- make step -- STEP -- Biman algorithm ----------------------- //

void Step(Atom a[], Atom out_atoms[], double dt)
{
  int i; double fx, fy;
  calc_derivatives(a, acel_2);
  for (i=1;i<=N;i++)
  {
  out_atoms[i].x = a[i].x + (a[i].vx+(4*acel_2[i].vx-acel_1[i].vx)/6*dt)*dt; //*1e10;
  out_atoms[i].y = a[i].y + (a[i].vy+(4*acel_2[i].vy-acel_1[i].vy)/6*dt)*dt; //*1e10;
  }
  calc_derivatives(out_atoms, acel_3);
  for (i=1;i<=N;i++)
  {
  out_atoms[i].vx = a[i].vx + (2*acel_3[i].vx+
  5*acel_2[i].vx-acel_1[i].vx)/6*dt;
  out_atoms[i].vy = a[i].vy + (2*acel_3[i].vy+
  5*acel_2[i].vy-acel_1[i].vy)/6*dt;
  acel_1[i].vx = acel_2[i].vx;
  acel_1[i].vy = acel_2[i].vy;
  }
 
  if (mode == Mode_Fixed)     //Fixed
   {
    for (i=1;i<=N;i++)
    {
      border_force(out_atoms[i].x, out_atoms[i].y, fx, fy);
      if (fx>=0) impulse_x = impulse_x + fx; else impulse_x - fx;
      if (fy>=0) impulse_y = impulse_y + fy; else impulse_y - fy;
    }
   }
  else  //non Fixed - Pereodic
    {
    for (i=1;i<=N;i++)
      {
      if (out_atoms[i].x > cellsize_x)
       {
        out_atoms[i].x = out_atoms[i].x - cellsize_x;
        initial_atoms[i].x = initial_atoms[i].x - cellsize_x;
        if (a[i].vx>=0) impulse_x = impulse_x + 2*a[i].vx;
          else impulse_x = impulse_x - 2*a[i].vx;
       }
      if (out_atoms[i].y > cellsize_y)
       {
        out_atoms[i].y = out_atoms[i].y - cellsize_y;
        initial_atoms[i].y = initial_atoms[i].y - cellsize_y;
        if (a[i].vy>=0) impulse_y = impulse_y + 2*a[i].vy;
          else impulse_y = impulse_y - 2*a[i].vy;
       }
      if (out_atoms[i].x < 0)
       {
        out_atoms[i].x = out_atoms[i].x + cellsize_x;
        initial_atoms[i].x = initial_atoms[i].x + cellsize_x;
         if (a[i].vx>=0) impulse_x = impulse_x + 2*a[i].vx;
          else impulse_x = impulse_x - 2*a[i].vx;
       }
      if (out_atoms[i].y < 0)
       {
        out_atoms[i].y = out_atoms[i].y + cellsize_y;
        initial_atoms[i].y = initial_atoms[i].y + cellsize_y;
        if (a[i].vy>=0) impulse_y = impulse_y + 2*a[i].vy;
          else impulse_y = impulse_y - 2*a[i].vy;
       }
      }
    }
}

void make_step(int steps, double dt)
{int i;
for (i=1;i<=steps;i++) {Step(atoms, atoms, dt); tim +=1;}
  total_time = total_time + dt * steps * t_unit;
}
        //-------------------------------|||--------------
void calc_energy(Atom a[],Energy EA[],
                      double &kmin,double &kmax,double &kave,
                      double &umin,double &umax,double &uave,
                      double &emin,double &emax,double &eave,
                      double &eetotal)
  { int i,j; double e; eetotal = 0;
    for  (i=1;i<=N;i++)   // kinetic en.,  
    { EA[i].k =(a[i].vx*a[i].vx+a[i].vy*a[i].vy)*0.5; 
      EA[i].u = 0;    // start for potential energy of each particle
      eetotal = eetotal + EA[i].k; // the total energy, kinetic yet
    } 
for  (i=1;i<=N-1;i++)
      for  (j=i+1;j<=N;j++)
    // calculation of potential energy of interaction of i-th and j-th particles
  {e = potential_direct(a[i].x-a[j].x, a[i].y-a[j].y);
   EA[i].u = EA[i].u + e; EA[j].u = EA[j].u + e;
       eetotal = eetotal + e;}	// adding of the energy

  	 if (mode == Mode_Fixed)
    for  (i=1;i<=N;i++)   
 //contribution to the potential energy due to interaction with elastic walls
    {e = border_potential(a[i].x, a[i].y);
     EA[i].u = EA[i].u + e; eetotal = eetotal + e; }
   else       // the case of periodic boundary conditions
      for  (i=1;i<=N-1;i++)
      for  (j=i+1;j<=N;j++)
   {e=potential_periodic(a[i].x-a[j].x,a[i].y-a[j].y);
    EA[i].u = EA[i].u + e; EA[j].u = EA[j].u + e;
      eetotal = eetotal + e; }  // adding of this contribution
   
/*below determination of average values and maximum and minimum of ener-gies for building of graphs*/
  kmin = 0; kmax = 0; kave = 0;
  umin = 0; umax = 0; uave = 0;
  emin = 0; emax = 0; eave = 0;
    for  (i=1;i<=N;i++)
 {e=EA[i].k;  if (e> kmax) kmax= e;  kave= kave + e;      
  e=EA[i].u;  if (e< umin) umin= e;  uave= uave + e;     
  e=EA[i].u+EA[i].k; if (e< emin) emin=e; eave=eave+e;
 }       // the average values of energies: kinetic, potential and total
  kave = kave/N;  uave = uave/N;  eave = eave/N;
}


// ----------------------- Model Interface ----------------------- //

void calc_GR(double &gmin,double &gmax,double &xmin,
                double &xmax)
{ int i,j,k,nr;  double d,d2,dr; int intR=40; int NCOUNT=0;
dr=potential_d/10;  /*the distant interval between two orbs -- d_atom/10  */  
  for (k=0;k<=100; k++) gr[k]=0;   //intial values zero

  for (i=0;i<=N-1;i++)
    for (j=i+1;j<=N;j++)
  {d2 = pow((atoms[i].x-atoms[j].x),2) + pow((atoms[i].y-atoms[j].y),2);
    d = sqrt(d2);     // the distance between two atoms
    nr = floor(d/dr); // the number of spherical layer
 if (nr>intR)continue; //intR*dr  is the radius of area of calculation
  gr[nr] += 1;   //accumulation of data on each layers, dr in thickness
  NCOUNT += 1;}		// the sum of all gr[nr]

  for (int k = 1; k <= intR; k++) 
 gr[k]=gr[k]/(pow(k+1,3)-pow(k,3))*pow(intR,3)/NCOUNT;
 xmin = 0; xmax = dr*intR; //xmax for graph is the size of the orb R;
   gmin = 0;   gmax = 0;		// for the scale of graph
  for (k=1;k<=intR; k++)
if (gr[k] > gmax) gmax = gr[k];  //the max value of gr[k]
}


void calc_nK(double nE[],double &nmin,double &nmax,double &e_min,double &e_max)
{
  Energy EA[N+1];
  double kmin, kmax, kave, umin, umax, uave, emin, emax, eave, etotal;
  int min_k, max_k, i, k;
  calc_energy(atoms, EA, kmin, kmax, kave, umin, umax, uave, emin, emax, eave, etotal);
  e_min = kmin;
  e_max = kmax;

  for (i=0;i<=Graph_N-1;i++) nE[i] = 0;
  nmin = 0;
  nmax = 0;
  for (i=1;i<=N;i++)
    {
    min_k = floor(Graph_N*((EA[i].k-e_min)/(e_max-e_min) - 0.5/E_Slices));
    max_k = ceil(Graph_N*((EA[i].k-e_min)/(e_max-e_min) + 0.5/E_Slices));
    if (min_k < 0) min_k = 0;
    if (max_k < 0) max_k = 0;
    if (min_k > Graph_N-1) min_k = Graph_N-1;
    if (max_k > Graph_N-1) max_k = Graph_N-1;
    for (k=min_k;k<=max_k;k++)
      {
      nE[k] = nE[k] + 1;
      if (nE[k] > nmax) nmax = nE[k];
      }
  }
  for (i=0;i<=Graph_N-1;i++) nE[i] = 100*nE[i]/N;
  nmin = 100*nmin/N;
  nmax = 100*nmax/N;
}

void calc_nU( double nE[], double &nmin,double &nmax,double &e_min,double &e_max)
{
  Energy EA[N+1];
  double kmin, kmax, kave, umin, umax, uave, emin, emax, eave, etotal;
  int min_k, max_k, i, k;
  calc_energy(atoms, EA, kmin, kmax, kave, umin, umax, uave, emin, emax, eave, etotal);
  e_min = umin;
  e_max = umax;

  for (i=0;i<=Graph_N-1;i++) nE[i] = 0;

  nmin = 0;
  nmax = 0;
  for (i=0;i<=N;i++)
    {
    min_k = floor(Graph_N*((EA[i].u-e_min)/(e_max-e_min) - 0.5/E_Slices));
    max_k = ceil(Graph_N*((EA[i].u-e_min)/(e_max-e_min) + 0.5/E_Slices));
    if (min_k < 0) min_k = 0;
    if (max_k < 0) max_k = 0;
    if (min_k > Graph_N-1) min_k = Graph_N-1;
    if (max_k > Graph_N-1) max_k = Graph_N-1;
    for (k=min_k;k<=max_k;k++)
      {
      nE[k] = nE[k] + 1;
      if (nE[k] > nmax) nmax = nE[k];
      }
    }
  for (i=0;i<=Graph_N-1;i++) nE[i] = 100*nE[i]/N;
  nmin = 100*nmin/N;
  nmax = 100*nmax/N;
}

void calc_nE(double nE[],double &nmin,double &nmax,double &e_min,double &e_max)
{
  Energy EA[N+1];
  double kmin, kmax, kave, umin, umax, uave, emin, emax, eave, etotal;
  int min_k, max_k, i, k;
  calc_energy(atoms, EA, kmin, kmax, kave, umin, umax, uave, emin, emax, eave, etotal);
  e_min = emin;
  e_max = emax;

  for (i=0;i<=Graph_N-1;i++) nE[i] = 0;

  nmin = 0;
  nmax = 0;
  for (i=0;i<=N;i++)
    {
    min_k = floor(Graph_N*((EA[i].u+EA[i].k-e_min)/(e_max-e_min) - 0.5/E_Slices));
    max_k = ceil(Graph_N*((EA[i].u+EA[i].k-e_min)/(e_max-e_min) + 0.5/E_Slices));
    if (min_k < 0) min_k = 0;
    if (max_k < 0) max_k = 0;
    if (min_k > Graph_N-1) min_k = Graph_N-1;
    if (max_k > Graph_N-1) max_k = Graph_N-1;
    for (k=min_k;k<=max_k;k++)
     {
      nE[k] = nE[k] + 1;
      if (nE[k] > nmax) nmax = nE[k];
     }
  }
  for (i=0;i<=Graph_N-1;i++) nE[i] = 100*nE[i]/N;
  nmin = 100*nmin/N;
  nmax = 100*nmax/N;
}

void calc_deltas(double &ave_dr2, double &ave_vx2, double &ave_vy2)
{ double dr2, vx2, vy2;
   int i;
  dr2 = 0;
  vx2 = 0;
  vy2 = 0;
  for (i=0;i<=N;i++)
  {
  dr2 = dr2 + pow((atoms[i].x-initial_atoms[i].x),2)+ pow((atoms[i].y-initial_atoms[i].y),2);
  vx2 = vx2 + atoms[i].vx*initial_atoms[i].vx;
  vy2 = vy2 + atoms[i].vy*initial_atoms[i].vy;
  }
  ave_dr2 = dr2/N;
  ave_vx2 = vx2/N;
  ave_vy2 = vy2/N;
}

void reset_model(double x1, double x2,double y1,double y2,double KE)
{
  int i, j, k, kx, ky;
  double dx, dy, sx, sy;
  double  v, phi, dv;
  double  kmin, kmax, kave, umin, umax, uave, emin, emax, eave;
  kx = int(sqrt(N));
  ky = ceil(N/kx);
  dx = (x2-x1)/(kx);
  dy = (y2-y1)/(ky);
  sx = x1 + dx/2;
  sy = y1 + dy/2;
  dv = sqrt(2*KE);
  i = 0;
  j = 0;
  for (k=1;k<=N;k++)
    {
    atoms[k].x = sx + dx*i;
    atoms[k].y = sy + dy*j;
    v = (rand_double(dv*0.9, dv*1.1)+rand_double(dv*0.9, dv*1.1)+
          rand_double(dv*0.9, dv*1.1)+rand_double(dv*0.9, dv*1.1))/4.0;
    phi = rand_double(0, 2*M_PI);
    atoms[k].vx = v*cos(phi);
    atoms[k].vy = v*sin(phi);
    initial_atoms[k].x = atoms[k].x;
    initial_atoms[k].y = atoms[k].y;
    initial_atoms[k].vx = atoms[k].vx;
    initial_atoms[k].vy = atoms[k].vy;
    j = j + 1;
    if (j >= ky)
      {
      j = 0;
      i = i + 1;
      }
  }
  calc_energy(atoms, EA, kmin, kmax, kave, umin, umax, uave, emin, emax, eave, initial_energy);
  impulse_x = 0;
  impulse_y = 0;
  total_time = 0;
}

// ----------------------- Graph Drawing ----------------------- //

void guess_grid(double min, double max,double &step,double &vmin,double &vmax)
{ double base;
  if ((max-min)>0) base = pow(10, floor(log10(max-min)));
    else base = pow(10, floor(log10(min-max)));
  vmin = floor(min/base)*base;
  vmax = ceil(max/base)*base;
  if ((vmax-vmin)>=0)
    {
    if ((vmax-vmin) <= 1.2 * base)
      {
      base = base / 5;
      vmin = floor(min/base)*base;
      vmax = ceil(max/base)*base;
      }
    }
    else
    {
    if ((vmin-vmax) <= 1.2 * base)
      {
      base = base / 5;
      vmin = floor(min/base)*base;
      vmax = ceil(max/base)*base;
      }
    }
  if ((vmax-vmin)>=0)
    {
    if ((vmax-vmin) <= 3.0 * base)
      {
      base = base / 2;
      vmin = floor(min/base)*base;
      vmax = ceil(max/base)*base;
      }
    }
    else
    {
    if ((vmin-vmax) <= 3.0 * base)
      {
      base = base / 2;
      vmin = floor(min/base)*base;
      vmax = ceil(max/base)*base;
      }
    }
  step = base;
}

void DrawGrid(TCanvas *Canvas,
              AnsiString xtitle,AnsiString ytitle,
              int &xmin, int &ymin, int &xmax, int &ymax,
              double xvalmin,double yvalmin,double xvalmax,double yvalmax)
{ double v, step;
  double xvmin, xvmax, yvmin, yvmax;
  int c;
  AnsiString s;
  Canvas->FillRect(Canvas->ClipRect);
    Canvas->MoveTo(xmin, ymax);
    Canvas->LineTo(xmin-4, ymax+10);
    Canvas->MoveTo(xmin, ymax); Canvas->LineTo(xmin+4, ymax+10);
    Canvas->MoveTo(xmin, ymin); Canvas->LineTo(xmin, ymax+9);

    Canvas->MoveTo(xmax, ymin); Canvas->LineTo(xmax-10, ymin+4);
    Canvas->MoveTo(xmax, ymin); Canvas->LineTo(xmax-10, ymin-4);
    Canvas->MoveTo(xmin, ymin); Canvas->LineTo(xmax-9, ymin);
    Canvas->TextOut(xmin+8, ymax, ytitle);
    Canvas->TextOut(xmax-5*xtitle.Length()-3, ymin+Canvas->Font->Height-8, xtitle);

    guess_grid(xvalmin, xvalmax, step, xvmin, xvmax);
    v = xvmin + step;
    while (v < xvmax)
    {
      c = int(xmin + (v-xvmin)*(xmax-xmin)/(xvmax-xvmin));
      s = FloatToStr(v);
      Canvas->TextOut(c-3*s.Length(), ymin+Canvas->Font->Height-8, s);
      Canvas->MoveTo(c, ymin-3); Canvas->LineTo(c, ymin+3);
      v = v + step;
    }

    guess_grid(yvalmin, yvalmax, step, yvmin, yvmax);
    v = yvmin + step;
    while (v < yvmax)
    {
      c = int(ymin + (v-yvmin)*(ymax-ymin)/(yvmax-yvmin));
        s = FloatToStr(v);      
      Canvas->TextOut(xmin+8, c-5, s);
      Canvas->MoveTo(xmin-3, c); Canvas->LineTo(xmin+3, c);
      v = v + step;
    }

    xmin = int(xmin + (xvalmin-xvmin)*(xmax-xmin)/(xvmax-xvmin));
    xmax = int(xmin + (xvalmax-xvmin)*(xmax-xmin)/(xvmax-xvmin));

    ymin = int(ymin + (yvalmin-yvmin)*(ymax-ymin)/(yvmax-yvmin));
    ymax = int(ymin + (yvalmax-yvmin)*(ymax-ymin)/(yvmax-yvmin));
}
                              ////////||||||||||
void DrawGraph(TCanvas *Canvas, double YY[],
               double yvmin,double yvmax,double xvmin,double xvmax,
               AnsiString ytitle,AnsiString xtitle)
{ int i, x, y, ymin, ymax, xmin, xmax;
  double xscale, yscale;
    ymin = Canvas->ClipRect.Bottom - 10;
    ymax = Canvas->ClipRect.Top + 10;
    xmin = Canvas->ClipRect.Left + 10;
    xmax = Canvas->ClipRect.Right - 10;
    Canvas->Brush->Color = RGB(255,255,255);
    Canvas->Pen->Color = RGB(0,0,0);
    Canvas->FillRect(Canvas->ClipRect);
    DrawGrid(Canvas, xtitle, ytitle, xmin, ymin, xmax, ymax, xvmin, yvmin, xvmax, yvmax);
    xscale = (xmax-xmin)/(40-0.5); // Graph_N
    yscale = (ymax-ymin)/(yvmax-yvmin);

    Canvas->Brush->Color = RGB(0,0,0);
    Canvas->Pen->Color = RGB(180,0,0);
    y = int(YY[0] * yscale + ymin);
    x = int(0.5 * xscale + xmin);
    Canvas->MoveTo(int(xmin), int(ymin));
    Canvas->LineTo(int(xmin), y);
    Canvas->LineTo(x, y);
    for(i=1;i<=GR_N-1;i++)
      {
      x = int((i-0.5) * xscale + xmin);
      y = int(YY[i] * yscale + ymin);
      Canvas->LineTo(x, y);
      x = int((i+0.5) * xscale + xmin);
      Canvas->LineTo(x, y);
    }
    Canvas->LineTo(int(xmax), int(ymin));
}

// ----------------------- User Interface code ----------------------- //

void __fastcall TForm1::QuitButtonClick(TObject *Sender)
{
Application->ProcessMessages();
Application->Terminate();
}
//---------------------------------------------------------------------------

void TForm1::RedrawImage()
{double ymin, ymax, xmin, xmax;
  DrawAtoms(AtomView->Canvas);
  if (AdditionalPages->ActivePageIndex == 1) UpdateStatistics();
   else
    {
    switch(AdditionalPages->ActivePageIndex)
    {
      case 2:   if (tim % 25 ==0)
           {calc_GR(/*gr,*/ ymin, ymax, xmin, xmax);
           DrawGraph(GRView->Canvas, gr, ymin, ymax, xmin, xmax,
                     "g(r)", "r/d");
          }
      case 3: {
           calc_nK(GR, ymin, ymax, xmin, xmax);
           DrawGraph(KEnergyView->Canvas, GR, ymin, ymax, xmin, xmax,
                     "P, %", "K, K");
          }
      case 4: {
           calc_nU(GR, ymin, ymax, xmin, xmax);
           DrawGraph(UEnergyView->Canvas, GR, ymin, ymax, xmin, xmax,
                     "P, %", "U, K");
          }
      case 5: {
           calc_nE(GR, ymin, ymax, xmin, xmax);
           DrawGraph(EnergyView->Canvas, GR, ymin, ymax, xmin, xmax,
                     "P, %", "E, K");
          }
     }
    }
}

void TForm1::DrawAtoms(TCanvas *Canvas)
{
  int i, x, y;
  double xscale, xshift, yscale, yshift;
    xscale = (Canvas->ClipRect.Right-Canvas->ClipRect.Left-20)/cellsize_x;
    yscale = (Canvas->ClipRect.Bottom-Canvas->ClipRect.Top-20)/cellsize_y;
    xshift = Canvas->ClipRect.Left+10;
    yshift = Canvas->ClipRect.Top+10;
    Canvas->Brush->Color = RGB(255,255,255);
    Canvas->Pen->Color = RGB(140,140,140);
    Canvas->FillRect(Canvas->ClipRect);
    Canvas->MoveTo(Canvas->ClipRect.Left+10,Canvas->ClipRect.Top+10);
    Canvas->LineTo(Canvas->ClipRect.Right-10, Canvas->ClipRect.Top+10);
    Canvas->LineTo(Canvas->ClipRect.Right-10, Canvas->ClipRect.Bottom-10);
    Canvas->LineTo(Canvas->ClipRect.Left+10, Canvas->ClipRect.Bottom-10);
    Canvas->LineTo(Canvas->ClipRect.Left+10, Canvas->ClipRect.Top+10);
    Canvas->Brush->Color = RGB(255,255,255);
    Canvas->Pen->Color = RGB(155,155,155);
    for (i=1;i<=N;i++)
    {
      x = int(atoms[i].x * xscale + xshift);
      y = int(atoms[i].y * yscale + yshift);
      Canvas->RoundRect(x-2, y-2, x+3, y+3, 2, 2);
    }
}

void TForm1::UpdateStatistics()
{ Application->ProcessMessages();
  double kmin, kmax, kave, umin, umax, uave, emin, emax, eave, etotal;
  double dr2, vx2, vy2;
  calc_energy(atoms, EA, kmin, kmax, kave, umin, umax, uave, emin, emax, eave, etotal);
  calc_deltas(dr2, vx2, vy2);
  TotalTimeView->Caption = FloatToStrF(total_time, ffGeneral, 10, 4);
  TotalEnergyView->Caption = FloatToStrF(etotal, ffGeneral, 10, 4);
  MinKEnergyView->Caption = FloatToStrF(kmin, ffGeneral, 6, 4);
  AveKEnegryView->Caption = FloatToStrF(kave, ffGeneral, 6, 4);
  MaxKEnergyView->Caption = FloatToStrF(kmax, ffGeneral, 6, 4);
  MinUEnergyView->Caption = FloatToStrF(umin, ffGeneral, 6, 4);
  AveUEnergyView->Caption = FloatToStrF(uave, ffGeneral, 6, 4);
  MaxUEnergyView->Caption = FloatToStrF(umax, ffGeneral, 6, 4);
  if (((etotal-initial_energy)/initial_energy)>=0)
    ErrorView->Caption = FloatToStrF((etotal-initial_energy)/initial_energy, ffExponent, 1, 1);
    else
    ErrorView->Caption = FloatToStrF((initial_energy-etotal)/initial_energy, ffExponent, 1, 1);
  DR2View->Caption = FloatToStrF(dr2, ffGeneral, 6, 4);
  if (total_time > 0)
  {
    DView->Caption = FloatToStrF(dr2/2.0/total_time, ffGeneral, 6, 4);
    XPressureView->Caption = FloatToStrF(impulse_x/cellsize_y/total_time, ffGeneral, 6, 4);
    YPressureView->Caption = FloatToStrF(impulse_y/cellsize_x/total_time, ffGeneral, 6, 4);
  }
    else
    {
    DView->Caption = "0/0";
    XPressureView->Caption = "0/0";
    YPressureView->Caption = "0/0";
    }
}

void __fastcall TForm1::FormCreate(TObject *Sender)
{
//randomize;
  mode = Mode_Fixed;
  initial_K = initial_kinetic;
  reset_model(initial_x1, initial_x2, initial_y1, initial_y2, initial_K);

  time_step = initial_dt;
  DecimalSeparator = '.';
  Running = False;
  ParticlesView->Caption = IntToStr(N);
  TimeStepEdit->Text = FloatToStr(time_step);
  SPF = 100;
  SPFEdit->Text = IntToStr(SPF);
  Potential_EEdit->Caption = FloatToStr(potential_e);
  Potential_DEdit->Caption = FloatToStr(potential_d);
  CellSizeXEdit->Caption = FloatToStr(cellsize_x);
  CellSizeYEdit->Caption = FloatToStr(cellsize_y);
  InitialKEdit->Text = FloatToStr(initial_K);
  Form1->RedrawImage();
}




//---------------------------------------------------------------------------

void __fastcall TForm1::StartButtonClick(TObject *Sender)
{
  if (mode==Mode_Fixed) PeriodicBorderSwitch->Enabled=false;
    else FixedBorderSwitch->Enabled=false;
  Running=!Running;
  if (Running) StartButton->Caption="Stop";
          else StartButton->Caption="Go";
  while (Running)
  {
  //total_time=total_time+StrToFloat(TimeStepEdit->Text);
   make_step(25, time_step);
  Form1->UpdateStatistics();
  Application->ProcessMessages();
  Form1->RedrawImage();
  Application->ProcessMessages();
  }
   Application->ProcessMessages();
}


//---------------------------------------------------------------------------

void __fastcall TForm1::OneFrameButtonClick(TObject *Sender)
{
 if (mode==Mode_Fixed) PeriodicBorderSwitch->Enabled=false;
    else FixedBorderSwitch->Enabled=false;
  Running=false;
  StartButton->Caption="Go";
  Application->ProcessMessages();
  make_step(25, time_step);
  Form1->UpdateStatistics();
  Application->ProcessMessages();
  Form1->RedrawImage();
  Application->ProcessMessages();
}
//---------------------------------------------------------------------------

void __fastcall TForm1::ResetButtonClick(TObject *Sender)
{
if (!Running)
  {
  PeriodicBorderSwitch->Enabled=true;
  FixedBorderSwitch->Enabled=true;
  }
if (Running)
  {
    Running=false;
    reset_model(initial_x1, initial_x2, initial_y1, initial_y2, initial_K);
    Form1->RedrawImage();
    Running=true;
  }
   else
   {
    reset_model(initial_x1, initial_x2, initial_y1, initial_y2, initial_K);
    Form1->RedrawImage();
   }
   Application->ProcessMessages();
}

//---------------------------------------------------------------------------

void __fastcall TForm1::PeriodicBorderSwitchClick(TObject *Sender)
{
  if (Running)
  {
    Running=false;
    Application->ProcessMessages();
    mode = Mode_Periodic;
    Running=true;
    Application->ProcessMessages();
  }
  else
  {
  mode = Mode_Periodic;
  Application->ProcessMessages();
  }
}


//---------------------------------------------------------------------------

void __fastcall TForm1::FixedBorderSwitchClick(TObject *Sender)
{
  if (Running)
  {
    Running=false;
    Application->ProcessMessages();
    mode = Mode_Fixed;
    Running=true;
    Application->ProcessMessages();
  }
  else
  {
  mode = Mode_Fixed;
  Application->ProcessMessages();
  }
}

//---------------------------------------------------------------------------

void __fastcall TForm1::SPFApplyButtonClick(TObject *Sender)
{
int steps;
steps = StrToInt(SPFEdit->Text);
SPF = steps;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::ApplyTimeStepButtonClick(TObject *Sender)
{
float dt;
dt = StrToFloat(TimeStepEdit->Text);
time_step = dt;
}


//---------------------------------------------------------------------------

void __fastcall TForm1::InitialKApplyButtonClick(TObject *Sender)
{
double K = StrToFloat(InitialKEdit->Text);
initial_K = K;
}
//---------------------------------------------------------------------------

